# Tutorial

## Application Structure
Below is the directory structure of a zavin-as application.  The figure is based in the `hello-world` app demonstrated in the _Quickstrat_ guide.

```
hello-world
    - public/
        - assets/
        - docs/
    - routes/
        - index.js
    - views/
        - index.html
    - config.json
    - index.js
    - readme.md
```

- `public/`. The `public` directory contains the application-specific assets such as custom CSS, scripts, fonts, images, etc.  This is where you could also put static pages such as application user guides or tutorials.
- `routes/`. The `routes` directory contains the routes which serves as the controller between the models and views in your app.
- `views/`. The `views` directory contains the pages used in your app.
- `config.json`. The application-specific configuration. Put other config here.
- `index.js`. This is the application bootstrap with provides initialization when app is registered.
- `readme.md`. This is where you'll describe about your app.



## Application Configuration
Each application contains its own `config.json` which allows you to define your application-specific preferences.  Be default, it contains a single property called `name` which is used to identify the application name.  This is also used in some parts of the application such as the URL of the app or the application title in the web page as shown in the browser's title bar.

``` json
{
  "name": "hello-world"
}
```

That is why in the sample `hello-world` application the URL to access your application would be `http://localhost:3000/hello-world`. Accessing information in the application `config.json` simply requires you to import in your routes such as

``` javascript
var config = require("../config.json");
```

Notice the relative path used to locate the file.

## Register your Application

Once you have your application, you need to register it in the `register.js` for located in the `./app/` directory. This makes all route components of your app avaialble to zavin-as. Below is the sample when we register our `hello-world` app.

``` javascript
/*
 * Register your apps below
 */

require("./default");
require("./hello-world");
```

## About Routes

Routes are the main components used to control processing request and rendering output to response such as the view (web page).

``` javascript
"use strict";
var router = module.exports = require("express").Router(),
    config = require("../config.json");

const url = `/${config.name}`;
const path = `${process.cwd()}/app/${config.name}/views`;

//- route: index/list
router.get(url, function(req, res) {
    res.locals.url = url;
    res.locals.user = req.user;
    res.locals.config = config;

    res.render(`${path}/index`);
});
```

We'll explain each section of the code.

``` javascript
"use strict";
```
The strict mode is a way to opt in to a restricted variant of JavaScript. This eliminates some JavaScript silent errors by changing them to throw errors.

``` javascript
var router = module.exports = require("express").Router(),
    config = require("../config.json");
```

Next, we define our router object by importing it from Express and at the same time as a module. We also import `config.json` to use the `name` property.

``` javascript
const url = `/${config.name}`;
```

Next we define the url property which represents the URL used to identify this route within the application. We use string interpolation which in the example earlier will translate to `/hello-world` as the URL of the app.

``` javascript
const path = `${process.cwd()}/app/${config.name}/views`;
```

The `path` property indicated the location of the view resources of our application. This is where zavin-as uses the path to render given pages.

``` javascript
//- route: index/list
router.get(url, function(req, res) {
    res.locals.url = url;
    res.locals.user = req.user;
    res.locals.config = config;

    res.render(`${path}/index`);
});
```

This responds to HTTP GET request from the given URL and displays the given web page resource. Notice how path is interpolated to indicate the location of the web page. Certain properties such as `url`, `user` and `config` are set to the response local variable which is made available to the view.

### Adding Routes

Below we'll add a sample route besides the one provided from the template. We'll continue from the sample `hello-world` app shown in the _Quickstart_ chapter.  First, we create the file `hello.js` in `routes/` directory:

Creating `./app/hello-world/routes/hello.js`
``` javascript
"use strict";
var router = module.exports = require("express").Router(),
    config = require("../config.json");

const url = `/${config.name}/hello`;
const path = `${process.cwd()}/app/${config.name}/views`;

//- route: index/list
router.get(url, function(req, res) {
    res.locals.url = url;
    res.locals.user = req.user;
    res.locals.config = config;

    res.render(`${path}/hello`);
});
```

The example above is similar from the previous code except to two things:

1. URL path is now `/hello-world/hello`
2. The response now renders `hello.html`

We'll be creating the view later.

## Bootstraping your Route

Once you have your route route ready, you need to bootstrap in order to use it in you application server.  To do this simply add it in the application bootstrap (index.js) file located in the application's root folder (`./app/{your_app}/index.js`). Just add it in the "Application Routes" section indicated in the comments. Your application bootstrap file should like something like the following:

``` javascript
/**
 * Application Routes
 * Import routes are defined here
 */
app.use(require("./routes"));
app.use(require("./routes/hello"));
```

We tell zavin-as to mount the specified router making it available for use. Notice that you don't have to include the `.js` extension. At this point if you try to access the address `localhost:3000/hello-world/hello` you should get a _**Server Error**_ with the message saying that it failed to lookup view. This is true as we don't have any hello.html yet.


## About Views

Views are the template view of your web application. In simple terms this are the web pages rendered from your routes. zavin-as provides the `./app/{your_app}/views` directory to put your templates. You can organize them by sub-directory, you just need to include the exact location in your route. The file is HTML5 template and uses [EJS](https://github.com/mde/ejs) for processing response data.

### Adding a View

We'll continue with our `hello-world` sample to add the hello.html file.  

Create the hello.html file into `./app/{your_app}/views` directory.

Creating `./app/hello-world/views/hello.html`

``` html
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title><%= config.name %></title>
</head>
<body>
    <h1>Hello, world</h1>
</body>
</html>
```

Below is a sample output

![hello](images/hello.jpg)

The template includes a sample `index.html` template that includes references to [Twitter Bootstrap](http://v4-alpha.getbootstrap.com/) and [jQuery](https://jquery.com/) libraries built with zavin-as.  This allow you to use the default libraries located in the global resource path `zavin-as/public/assets/`. Note that zavin-as uses Bootstrap v4.0.0-alpha.2. Please refer to the reference [site](http://v4-alpha.getbootstrap.com/getting-started/introduction/).

## About Assets
- Global Assets
- Local Assets
- Third Party Libraries

## Application Documentation

## Application Bootstrap
## Application Configuration
