/*
 * flash-message.js
 * Copyright © 2015 CNK Dynamics Co.
 */
"use strict";

/**
 * The FlashMessage class provides convenience methods for connect-flash.
 * This requires to install 'connect-flash'. Flash contains 3 properties:
 *  1. type - the type of message ie. 'info' or 'error'
 *  2. message - The message
 *  3. data - further data
 *
 * @example <caption>Store message/input which is avaialble in views</caption>
 * var flashMessage = require(process.cwd() + "/lib/flash-message");
 *     ...
 * router.get(uri + "/new", function(req, res) {
 *     res.locals.title = title;
 *     res.locals.flash = flashMessage.get(req);
 *     ...
 *     res.render(path + "create");
 * }
 *
 * @example <caption>Optionally, include Person instance with default properties</caption>
 * var flashMessage = require(process.cwd() + "/lib/flash-message");
 *     ...
 * router.get(uri + "/new", function(req, res) {
 *     res.locals.title = title;
 *     res.locals.flash = flashMessage.get(req, new Person({}));
 *     ...
 *     res.render(path + "create");
 * }
 *
 * @example <caption>Use flash in data validation messages</caption>
 * var flashMessage = require(process.cwd() + "/lib/flash-message");
 * var Person = require("zavin-core");
 *     ...
 * router.post(uri + "/new", function(req, res) {
 *     if (!req.body.foo) {
 *         res.status(422);
 *         flashMessage.error(req, 'Oops! Foo required', req.body);
 *         return res.redirect(uri + "/new");
 *     }
 *     // else ok to post
 *     ...
 * }
 *
 * @example <caption>Set HTML input 'value' attribute with flash.data</caption>
 * &lt;form action="..." method="post"&gt;
 *     &lt;input name="foo" value="#{flash.data.foo}"&gt;
 *     ...
 * &lt;/form&gt;
 */

class FlashMessage {

    /**
     * Set the content of flash as an info message
     *
     * @param  {object} req     The request object
     * @param  {string} message The message
     * @param  {object} data    Optional data
     */
    static info (req, message, data) {
        if (!req || !message) {
            throw new Error("Invalid parameter");
        }
        return req.flash("_flash_", ["info", message, data || {}]);
    }

    /**
     * Set the content of flash as an error message
     *
     * @param  {object} req     The request object
     * @param  {string} message The message
     * @param  {object} data    Optional data
     */
    static error (req, message, data) {
        if (!req || !message) {
            throw new Error("Invalid parameter");
        }
        return req.flash("_flash_", ["info", message, data || {}]);
    }

    /**
     * Get flash. The 'data' can be set as default {}
     *
     * @param  {object} req     The request object
     * @param  {object} data    Optional data
     */
    static get (req, data) {
        var defaultValue = data || {};
        var flash = req.flash("_flash_");

        return {
            type: flash[0] || "",
            message: flash[1] || "",
            data: flash[2] || defaultValue
        };
    }
}

module.exports = FlashMessage;

//- end
