/*
 * Copyright © 2016 CNK Dynamics Co.
 */
"use strict";
var bodyParser      = require("body-parser"),
    browserSync     = require('browser-sync'),
    compression     = require("compression"),
    cookieParser    = require("cookie-parser"),
    express         = require("express"),
    expressSession  = require("express-session"),
    flash           = require("connect-flash"),
    forceSSL        = require("express-force-ssl"),
    fsExtra         = require("fs-extra"),
    helmet          = require("helmet"),
    http            = require("http"),
    https           = require("https"),
    MongoClient     = require("mongodb").MongoClient,
    MongoStore      = require("connect-mongo")(expressSession),
    morgan          = require("morgan"),
    override        = require("method-override"),
    passport        = require("passport"),
    path            = require("path"),
    rateLimit       = require('express-rate-limit'),
    serveFavicon    = require("serve-favicon"),
    winston         = require("winston"),
    pkg             = require("./package.json");


let homeEnv = (process.env.HOME || (process.env.HOMEDRIVE + process.env.HOMEPATH)),
    homeDir = homeEnv + "/.zavin/",
    logDir  = homeDir + "logs";


/*
 * Setup log directory. The logDir defines the path to the log folder. It
 * ensures the directory exists.  Otherwise, create it.
 */
fsExtra.ensureDirSync(logDir);


/*
 * Create the exportable application object
 */
var app = module.exports = express();


/*
 * Setup console and file loggers
 */
let logger = new (winston.Logger)({
    exitOnError: false,
    transports: [
        new (winston.transports.Console)({
            level: "info",
            handleExceptions: true,
            colorize: true
        }),
        new (winston.transports.File)({
            level: "info",
            filename: pkg.name + ".log", // name of the log file
            dirname: logDir, // path to log file
            handleExceptions: true,
            json: true,
            maxsize: 1024000, // 1MB limit to log file
            maxFiles: 5,
            colorize: false
        })
    ]
});


//
// Show some stdout info
//
logger.info("%s v%s", pkg.name, pkg.version);
logger.info("home dir: %s...", homeDir);
logger.info("logs dir: %s...", logDir);


/*
 * System Configuration File
 * This will check if user-defined config is set in the home environment
 * directory {homeDir}. Otherwise, the server will use the default located
 * in etc directory. This will be available via app.get("config")
 */
let config, configPath, configFile = "config.json";
try {
    configPath = homeDir + configFile;
    fsExtra.statSync(configPath); // check stat
} catch (e) {
    configPath = "./" + configFile;
} finally {
    config = require(configPath);
    app.set("config", config);
    logger.info("Using config " + configPath);
}


//
// Application common settings
//
app.set("trust proxy", true); // to get/use req.ip
app.set("x-powered-by", false); // disable "X-Powered-By"
app.set("secret-key", pkg.name); // key for encryption and authentiction
app.set("logger", logger); // set reference to logger
app.set("title", pkg.title || pkg.name); // set default app title
app.set("version", pkg.version); // set default app title
app.set("homeDir", homeDir);


/*
 * Setup EJS as template with HTML extension
 */
app.set("views", [path.join(__dirname, "public")]);
app.engine('html', require('ejs').renderFile);
app.set("view engine", "html");


/*
 * Middleware: express-force-ssl
 * Setup SSL/TLS. This are options used when SSL is enabled. This will read the
 * SSL certificate, set in the config file the define the SSL option, then
 * register the middleware to force HTTPS
 */
var sslOptions;
if (config.enable_ssl) {
    let sslKey, sslCert;

    // Read certificate, key
    try {
        sslKey = fsExtra.readFileSync(config.signed_key_path, "utf8");
        sslCert = fsExtra.readFileSync(config.signed_cert_path, "utf8");
    } catch (err) {
        logger.error("Error accessing certificate:", err.message);
        process.exit(1);
    }

    // SSL option for HTTPS server
    sslOptions = {key: sslKey, cert: sslCert};

    // Setup forceSSL options
    app.set("forceSSLOptions", {
        enable301Redirects: true,
        trustXFPHeader: false,
        httpsPort: config.port_ssl,
        sslRequiredMessage: "SSL/TLS Required."
    });

    app.use(forceSSL); // Enable force SSL option
}


/*
 * Database URL
 * This defines the database connection and port settings. It also checks setup
 * for replica and public hosting. "public" environment variable should be set
 * as optional setting when deployed such as in Heroku. This is useful to have
 * three different conifigurations for local, public and replica set.
 */
var dburl;
if (app.get("env") === "development") {

    app.use(morgan("dev"));

    app.set("port", process.env.PORT || config.port_dev);
    dburl = config.datasource_dev;
} else {
    app.set("port", process.env.PORT || config.port_prod);
    dburl = config.datasource_prod;
}


/*
 * Middleware: method-override
 * Use HTTP verbs (PUT/DELETE) where client doesn’t support it
 */
app.use(override("_method"));


/*
 * Middleware: serve-favicon
 * Serve favicon with caching
 */
app.use(serveFavicon(__dirname + "/public/assets/images/logo.png"));


/*
 * Middleware: body-parser
 * Parse form data setting to 'req.body' property. This does not include
 * multipart form data. See formidable instead.
 */
app.use(bodyParser.json({limit: "50mb"}));
app.use(bodyParser.urlencoded({limit: "50mb", extended: true}));


/*
 * Middleware: cookie-parser
 * Cookie parsing with signatures
 */
app.use(cookieParser());


/*
 * Middleware: compression
 * Compress requests that pass through providing smaller data transfer
 */
app.use(compression());


/*
 * Middleware:
 * Indicate the location of static files/pages
 */
app.use(express.static(path.join(__dirname, "public")));


/*
 * Middleware: express-session
 * Simple session middleware. It uses mongodb to store session
 * This reuses the native MongoDB driver connection
 */
app.use(expressSession({
    resave: false,
    saveUninitialized: false,
    secret: pkg.name + pkg.version,
    store: new MongoStore({url: dburl})
}));


/*
 * Middleware: connect-flash
 */
app.use(flash());


/*
 * Middleware: passport, passport-local
 * Setup local authentication
 */
app.use(passport.initialize()); // initialize passport
app.use(passport.session()); // use session in passport


/*
 * Middleware: helmet
 * Secure your Express apps by setting various HTTP headers.
 * This enforces HTTP strict transport
 */
app.use(helmet.hsts());


/*
 * Middleware: express-rate-limit
 * Basic IP rate-limiting middleware which limit repeated requests
 */
app.use(rateLimit({delayAfter: 0, delayMs: 0, max: 0}));


/*
 * Connect to DB server
 */
MongoClient.connect(dburl, function(err, db) {

    if (err) {
        logger.error(err.message);
        process.exit(1);
    }

    /*
     * Set reference to db
     */
    app.set("db", db);


    /*
     * Routers:
     * This registers application routers. See the app/register.js
     */
    require("./app/register");


    /*
     * Error Handler: 404 'Not Found'
     * This will simply set status then forward to next error handler
     */
    app.use(function(req, res, next) {
        var err = new Error("Not Found");
        err.status = 404;
        next(err);
    });


    /*
     * Error Handler: 500 'Internal Server Error'
     * The last of the callback chain which handles internal errors
     */
    // jshint unused:false, es5:false
    app.use(function(err, req, res, next) {

        // print error stack if "development" else undefined
        err.stack = (app.get("env") === "development") ? err.stack : undefined;

        // check http status else use default
        err.status = err.status || 500;

        // set HTTP response status
        res.status(err.status);

        // If status is '404' show 404.html. Else, 500.html
        res.render(err.status === 404 ? "404" : "500", {err: err});
    });


    /*
     * Use Browsersync to auto-reload of pages when changes are made. This is
     * available only during development. If SSL/TLS is enabled proxy will use
     * HTTPS. Otherwise HTTP.
     */
    if ((app.get("env") === "development") && (config.enable_browser_sync)) {
        browserSync({
            browser: "safari",
            proxy: config.enable_ssl ? "https://localhost:" + config.port_ssl : "http://localhost:" + app.get("port"),
            files: [
                "app/*/*.{css}",
                "app/*/*.{html}",
                "app/*/*.{js}",
                "public/*/*.{css}",
                "public/*/*.{html}",
                "public/*/*.{js}"
            ]
        });
    }


    /*
     * Entry point:
     * Listen to incoming request. If SSL/TLS is enabled listens to both
     * HTTP and HTTPS requests. Otherwise, this will listen to HTTP only.
     */
    if (config.enable_ssl) {
        http.createServer(app).listen(app.get("port"), function () {
            logger.info("Running %s on port %s in %s mode via HTTP", pkg.name, app.get("port"), app.get("env"));
            https.createServer(sslOptions, app).listen(config.port_ssl, function () {
                logger.info("Running %s on port %s in %s mode via HTTPS", pkg.name, config.port_ssl, app.get("env"));
            });
        });
    } else {
        http.createServer(app).listen(app.get("port"), function () {
            logger.info("Running %s on port %s in %s mode via HTTP", pkg.name, app.get("port"), app.get("env"));
        });
    }


    /*
     * Event handler which initiate server shutdown
     */
    process.on("SIGINT", function () {
        logger.info("Stopping %s", pkg.name);
        process.exit(0);
    });


    /*
     * Event handler with SIGTERM to cause a program termination
     */
    process.on("SIGTERM", function () {
        logger.info("Terminating %s", pkg.name);
        process.exit(0);
    });
});

//- end
